#!/usr/bin/env python

__author__ = 'Matthias Muntwiler'

"""
@package loess.setup
setup.py file for LOESS

the LOESS code included here was developed at Bell Labs by
William S. Cleveland, Eric Grosse, Ming-Jen Shyu,
and is dated 18 August 1992.
the code is available in the public domain
from http://www.netlib.org/a/dloess.
see the README file for details.

the Python wrapper was set up by M. Muntwiler
with the help of the SWIG toolkit
and other incredible goodies available in the Linux world.

@bug this file is currently not used because
distutils does not compile the included Fortran files.

@author Matthias Muntwiler

@copyright (c) 2015 by Paul Scherrer Institut @n
Licensed under the Apache License, Version 2.0 (the "License"); @n
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
  http://www.apache.org/licenses/LICENSE-2.0
"""

from distutils.core import setup, Extension
from distutils import sysconfig

import numpy
try:
    numpy_include = numpy.get_include()
except AttributeError:
    numpy_include = numpy.get_numpy_include()
    
loess_module = Extension('_loess',
                         sources=['loess.i', 'loess_wrap.c', 'loess.c', 'loessc.c', 'predict.c', 'misc.c', 'loessf.f', 
                         'dqrsl.f', 'dsvdc.f'],
                         include_dirs = [numpy_include],
                         libraries=['blas', 'm', 'f2c'],
                         )

setup(name='loess',
      version='0.1',
      author=__author__,
      author_email='matthias.muntwiler@psi.ch',
      description="""LOESS module in Python""",
      ext_modules=[loess_module],
      py_modules=["loess"], requires=['numpy']
      )
